#!/usr/local/bin/perl -w
#
#   webstat.pl 5.12.00
#
#   Extract information from web access log file
#
#   Digicomp AG
#   Pirmin Kalberer <pka@sourcepole.ch>
#   
#   Example Usage: webstat.pl access.log | tee webstat.html
#
#   Similar programs: http://freshmeat.net/search/?q=apache+log+perl
#

use strict;

my $title = 'Logfile Analysis';
my $author = 'pka@sourcepole.ch';

my $searchpattern = 'cgi-bin/search.cgi\?Search=(.+) ';

# Global data structures:

# $hosts{$address} = $count
my %hosts;
keys(%hosts) = 1024; # allocate hash table

# $urls{$url} = $count
my %urls;

# $searches{$arg} = $count
my %searches;

# $statuscnt{$status} = $count
my %statuscnt;

my $hits = 0;
my $unparsed_lines = 0;


my %statusmap = 
  ( 200 => 'Success',
    202 => 'Accepted',
    203 => 'Partial Information',
    204 => 'No response',
    301 => 'Document moved',
    302 => 'Forward',
    304 => 'Not Modified',
    400 => 'Bad request',
    401 => 'Unauthorized',
    402 => 'Payment Required',
    403 => 'Forbidden',
    404 => 'Not Found',
    500 => 'Internal Server Error',
    501 => 'Not Implemented'
  );

# ------- Parse Logfile ---------
while (<>) {
  #Match full log line
  my ($host, $remote_user, $auth_user, $date, $request, $status, $bytes, $referral, $agent) = 
    /^(\S+) (\S+) (\S+) \[(.+?)\] \"(.*?)\" (\S+) (\S+) \"?(.*?)\"? \"(.*?)\"$/;
    
  if (! $host) {
    # Full version didn't match, try the short one:
    ($host, $remote_user, $auth_user, $date, $request, $status, $bytes) = 
      /^(\S+) (\S+) (\S+) \[(.+?)\] \"(.*?)\" (\S+) (\S+)/;
  }

  if ($host) {
    $hits++;
    
    $hosts{$host}++;
    
    $statuscnt{$status}++;
    
    if ($request =~ /\S+ (\S+) \S+/) {
    	$urls{$1}++;
    }

    if ($request =~ /$searchpattern/) {
    	$searches{$1}++;
    }
  } else {
    $unparsed_lines++;
  }
}


# ------- Result Output ---------
print "<HTML>
<HEAD>
<TITLE>$title</TITLE>
</HEAD>
<H1>$title</H1>
\n";

my ($key, $cnt);

print "<H2>Total Hits: $hits</H2>\n";

if ($unparsed_lines > 0) {
    print "<i>Unparsed lines: $unparsed_lines</i>\n";
}

print "<H2>HTTP Return Status</H2>\n<pre>\n";
foreach $key (keys %statuscnt) {
    print &getStatusText($key), ": $statuscnt{$key}\n";
}
print "</pre>\n";

print "<H2>Search Terms</H2>\n<pre>\n";
foreach $key (sort keys %searches) {
    print "$key\n";
}
print "</pre>\n";

print "<H2>Top Ten URLs</H2>\n<pre>\n";
$cnt = 0;
foreach $key (sort { $urls{$b} <=> $urls{$a} } keys %urls) {
    print "$urls{$key} $key\n";
    last if ++$cnt >= 10;
}
print "</pre>\n";

print "<H2>Hosts</H2>\n<pre>\n";
$cnt = 0;
foreach $key (keys %hosts) {
    print "$key ($hosts{$key} hits)\n";
    if (++$cnt >= 20) {
    	print keys(%hosts)-$cnt, " more...\n";
	last;
    }
}
print "</pre>\n";

print "
<BR>
<BR>
<ADDRESS>
<A HREF = \"mailto:$author\">$author</A>
</ADDRESS>
</BODY>

</HTML>
\n";


sub getStatusText {
  my ($status) = @_;
  
  my $statustext = $statusmap{$status};
  return $statustext ? $statustext : "Unknown ($status)";
}
